import java.util.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class ListManagerServlet extends HttpServlet {

   private Set addresses;

   private static class SubscriptionException extends Exception {
      SubscriptionException( String details ){ super(details); }
   }

   private synchronized void subscribe (String email)
   throws IOException, SubscriptionException {
      if  ( addresses.add (email) )   save();
      else throw new SubscriptionException(email+" was already subscribed.");
   }

   private synchronized void unsubscribe (String email)
   throws IOException, SubscriptionException { 
      if( addresses.remove(email) )   save();
      else throw new SubscriptionException(email+" was not subscribed.");
   } 

   private String filename;

   private void save() throws IOException {
      ObjectOutputStream out
            = new ObjectOutputStream(new FileOutputStream(filename));
      out.writeObject(addresses);
      out.close();
   }
 
   private void load() throws Exception {
        ObjectInputStream in 
	    = new ObjectInputStream(new FileInputStream(filename));
        addresses = (Set) in.readObject(); 
        in.close();
   } 

   public void init () throws ServletException {
       filename = getInitParameter("addressfile");
       if ( filename == null )     
          throw new UnavailableException ( this,
                  "The \"addressfile\" property must be set to a file name" );
       try {
          try {   load();  }
          catch ( FileNotFoundException e )  { 
              addresses = new HashSet();
              save();
	  }
       } catch ( Exception e ) { 
          throw new UnavailableException ( this,
                                       "Error accessing address file: " + e );
       } 
   } 

   protected void doGet( HttpServletRequest req, HttpServletResponse res )
   throws ServletException, IOException {
       res.setContentType("text/html");
       res.setHeader("pragma", "no-cache");
       PrintWriter out = res.getWriter();
       out.print("<HTML><HEAD><TITLE>ListManager</TITLE></HEAD>");
       out.print("<BODY><H3>Members:</H3><UL>");
       synchronized (this) {
          for ( Iterator i = addresses.iterator();  i.hasNext(); ) {
             out.print( "<LI>" ); 
             out.print( i.next() );
          }
       }
       out.print("</UL><HR><FORM METHOD=POST>");
       out.print("Enter your email address: <INPUT TYPE=TEXT NAME=email><BR>");
       out.print("<INPUT TYPE=SUBMIT NAME=action VALUE=subscribe>");
       out.print("<INPUT TYPE=SUBMIT NAME=action VALUE=unsubscribe>");
       out.print("</FORM></BODY></HTML>");
       out.close();
   }

   protected void doPost ( HttpServletRequest req, HttpServletResponse res )
   throws ServletException, IOException {
      try {
         String mes;
         String email = req.getParameter("email"); 
         if (email == null )
            throw new SubscriptionException("No email address given.");
         if ( "subscribe".equals(req.getParameter("action")) ){
            subscribe(email);
            mes = " has been subscribed";
         } else {
            unsubscribe(email);   
            mes = " has been removed";
         }
         printHTML(req,res,email,mes);
      } catch (SubscriptionException e) {
         res.sendError(res.SC_BAD_REQUEST, e.getMessage());
      }
   } 

   private void printHTML( HttpServletRequest req , HttpServletResponse res,
                           String email, String mes) throws IOException {
      res.setContentType("text/html");
      res.setHeader("pragma", "no-cache");
      PrintWriter out = res.getWriter();
      out.print("<HTML><HEAD><TITLE>List Manager</TITLE></HEAD><BODY>");
      out.print(email);
      out.print(mes);
      out.print("<HR><A HREF=\"");
      out.print(req.getRequestURI());
      out.print("\">Show the list</A></BODY></HTML>");
      out.close(); 
   }

   public String getServletInfo() {
      return "ListManagerServlet 1.1 by Stefan Zeiger, modified by KTO";
   }

}

